## This R code is used to monitor nation-wide HFMD in China from 2008-12-01 to 2009-12-31


### 1. Estimate the baseline model f(t)
##################################################################################################################

# calculate incidence rate
prvdat$ratio<-prvdat$counts/prvdat$population 
# in-control data
baselineprv<-subset(prvdat,Province == "Guizhou"|Province == "Sichuan"|
                      Province == "Yunnan" | Province == "Xinjiang" | Province == 'Tibet')

# estimated baseline incidence rate
baselineratio <- tapply(baselineprv$ratio,baselineprv$date_onset,mean)  
ratio <- baselineratio[1:365]

# 10-fold cross validation to search bandwidth and estimate incidence rate
span.sim <- NULL
for (i in c(1:1000)){
  span.sim[i] <- loess.wrapper(seq(1:length(ratio)),ratio, span.vals = seq(0.05, 0.3, by = 0.01), folds = 10)$span
}

span.vals <- mean(span.sim)    # the bandwidth 10-fold cv is around 0.1 
loess_fit <- loess.wrapper(seq(1:length(ratio)),ratio, span.vals = span.vals, folds = 10)
loess_fit$span  
loess_fit <- loess_fit$fit
pred_loess_ratio <- predict(loess_fit)



### 2. Fit model for overall Country
##################################################################################################################

basecount<-pred_loess_ratio*1337772                     # estimated incidence rate times population in the U.S.
detrenddata<-(nat_dat$TtCounts[336:700]-basecount)      # detrend the observed count data
IC_obs<-80    
len<-length(detrenddata)

# fit the ARIMA time series model
fit.ts <- auto.arima(detrenddata[1:80],max.p=10,max.q=10,stationary=F,allowdrift=F,ic=c("aicc"))
detrend.mean <- mean(detrenddata[1:IC_obs])
order.ts <- arimaorder(fit.ts)
ts.ml <- arima(detrenddata[1:80] - detrend.mean ,order = order.ts,include.mean=F,method = "CSS")
coeff <- ts.ml$coef
print(coeff)
est.res <- Forecast1(diff(detrenddata - detrend.mean), coeff, c(1,0,0))$residuals
est.res[1:79] <- ts.ml$res[2:80]
mean.res <- mean(ts.ml$res[2:80]) 
sd.res <- sd(ts.ml$res[2:80])
std.est.res <- (est.res-mean.res)/sd.res
shapiro.test((std.est.res[1:79])[-10])   # the residuals are tested to be normal  


### 3. Online monitoring
##################################################################################################################

k <- 1
h0 <- xcusum.crit(k,200, mu0 = 0, sided = "one")
C <- NULL
C0 <- 0; 
C[81] <- max(0,C0+std.est.res[(81-1)]-k);
for (i in 82:364){
  
  C[i] <- max(0,C[i-1]+std.est.res[(i-1)]-k)

}               
which(C > h0)  


