
## This R code is used to monitor state-level HFMD in China from 2008-12-01 to 2009-12-31 for each province

prvdat$ratio<-prvdat$counts/prvdat$population  
Prov<-unique(prvdat$Province)

### 1. We assume the first 80 observations for each state are in-control, their standard residuals are computed 
### one by one by detrending the baseline model and fitting the time series model. The standard residuals are
### stored in the vector std.residual.
##################################################################################################################

std.residual <- NULL

  p <- 1  # p is the index for the province, p = 1,...,31
  print(Prov[p])
  data <- (subset(prvdat,Province==Prov[p]))[1:365,]
  basecount <- pred_loess_ratio*data$population[1:365]
  detrend <- data$counts-basecount
  fit.ts <- auto.arima(detrend[1:80],max.p=10,max.q=10,stationary=F,allowdrift=F,ic=c("aicc"))
  order.ts <- arimaorder(fit.ts)
  detrend.mean <- mean(detrend[1:80])
  ts.ml <- arima(detrend[1:80] - detrend.mean,order=order.ts,include.mean=F,method = "CSS") 
  coeff <- ts.ml$coef
  est.res <- Forecast1(diff(detrend - detrend.mean), coeff, (order.ts - c(0,1,0)))$residuals
  # est.res <- Forecast1(diff(detrend - detrend.mean), coeff, order.ts)$residuals
  # The order of Forecast1 need to be adjusted for each province 
  est.res[1:79] <- ts.ml$res[2:80]
  est.res<-(est.res-mean(est.res[1:79]))/sd(est.res[1:79])
  std.residual[(79*(p-1)+1) : (79*p)] <- est.res[1:79]


### 2.Bootstrap procedure to search the control limit for a given k0 to reach the prespecified ARL0 level. 
### k is the allowance constant is set to be 0.25, 0.5, 1 in the paper. ARL0 is 200 and 500.
##################################################################################################################

RL <- NULL
sample_num <- 2000
k0 <- 1
h0 <- 3.9
for (j in 1:5000){
  rand <- sample(std.residual,sample_num,replace=T)
  Cn <- NULL
  C0 <- 0; 
  Cn[1] <- max(0,C0 + rand[1] + rnorm(1,0,0.01) - k0);   # plus a random number
  i <- 2;
  repeat{
    if(i > sample_num) {break(); j = j-1; print("miss");}
    Cn[i] <- max(0,Cn[i-1] + (rand[i] + rnorm(1,0,0.01)) - k0)
    if(Cn[i] > h0) {A = i;break();}else i <- i+1 
  }
  RL[j] <- A
}
mean(RL);sd(RL)


### 3. online monitoring the HFMD disease for each province by using CUSUM chart 
##################################################################################################################

for (p in c(1:31)){
  
  print(Prov[p])
  data <- (subset(prvdat,Province==Prov[p]))[1:365,]
  basecount <- pred_loess_ratio*data$population[1:365]
  detrend <- (data$counts-basecount)
  fit.ts <- auto.arima(detrend[1:80],max.p=10,max.q=10,stationary=F,allowdrift=F,ic=c("aicc"))
  order.ts <- arimaorder(fit.ts)
  detrend.mean <- mean(detrend[1:80])
  ts.ml <- arima(detrend[1:80] - detrend.mean,order=order.ts,include.mean=F,method = "CSS") 
  coeff <- ts.ml$coef
  est.res <- Forecast1(diff(detrend - detrend.mean), coeff, (order.ts - c(0,1,0)))$residuals
  # est.res <- Forecast1(diff(detrend - detrend.mean), coeff, order.ts)$residuals
  # The order of Forecast1 need to be adjusted for each province
  est.res[1:79] <- ts.ml$res[2:80]
  est.res<-(est.res-mean(est.res[1:79]))/sd(est.res[1:79])
  std.residual[(79*(p-1)+1) : (79*p)] <- est.res[1:79]
  
  # CUSUM chart
  k<-1
  h0<- 3.8
  C<-NULL
  C0 <- 0; 
  C[1]<-max(0,C0+est.res[80]-k);
  for (i in 2:365){
    
    C[i]<-max(0,C[i-1]+est.res[(79+i)]-k)
  }               
  print((which(C>h0))[1])
  
}