## Simulation for evaluating the performance in terms of false positive rate (FPR) and false negative rate (FNR). 

### 1. plot the in-control baseline count data.
##################################################################################################################
t <- seq(1:365)
# baseline of in control data
cos.t<- 0.2*cos(2*pi*t/365)
sin.t <- -0.1*sin(2*pi*t/365)
baseline <- 10 + 10*exp(-(t-150)^2/1000) + cos.t + sin.t
pop <- 200  # population
plot(t, baseline/pop, type='l')

### 2. Evaluate the false positive rate with 5000 replications.
### Simluate the errors e that follow a time series model ARIMA (1,0,0).
##################################################################################################################

FPR <- NULL  # False positive rate

for (sim in 1:5000){    # sim is simulation time for IC data
  set.seed(1999+sim)
  e <- arima.sim(list(order = c(1,0,0),ar=0.5), n = 365, sd = sqrt(1))
  
  # observed counts 
  counts <- as.numeric(ceiling(baseline + e))
  rate <- as.numeric(counts/pop)
  # cross validation to select bandwidth
  span <- loess.wrapper(seq(1:length(rate)),rate, span.vals = seq(0.2, 0.5, by = 0.01), 10)$span 
  IC.loess.fit <- loess(rate ~ t,span=span)
  detr.data <- rate -  IC.loess.fit$ fitted
  fit.ts <- auto.arima(detr.data,max.p=3,max.q=3,stationary=T,allowdrift=F,ic=c("aicc"))
  order.ts <- arimaorder(fit.ts)
  coeff <- arima(detr.data,order = order.ts,include.mean=F,method = "CSS")$coef
  IC.residuals <- arima(detr.data,order = order.ts, include.mean=F,method = "CSS")$residuals
  ICmean <- mean(IC.residuals) ;  ICsd <- sd(IC.residuals)


# Phase II online Monitoring

# Phase II in-control data simulation: simulate 365 IC data and calculate false positive rate
  e<- arima.sim(list(order = c(1,0,0),ar=0.5), n = 365, sd = sqrt(1))
  IC.sim.rate<- as.numeric(ceiling(baseline + e)/pop)
  detrend.IC.sim.rate<-IC.sim.rate - IC.loess.fit$fitted
  IC.sim.residuals <- Forecast1(detrend.IC.sim.rate, coeff, order.ts)$residuals
  sdt.IC.sim.residuals <- (IC.sim.residuals-ICmean)/ICsd
  
  # online monitoring
  k<-1
  h0<-xcusum.crit(k,500, mu0 = 0, sided = "one")
  C<-NULL
  C0 <- 0; 
  C[1]<-max(0,C0 + sdt.IC.sim.residuals[1]-k);
  for (i in 2:365){
    C[i]<-max(0,C[i-1]+sdt.IC.sim.residuals[i]-k)
  }  

  FPR[sim] <- length(which(C>h0))/365
  
} 
# The averaged false positive rate
  mean(FPR)  
  sd(FPR)



### 3. Evaluate the false negative rate with 5000 replications and different shift size delta.
### Simluate the errors e that follow a time series model ARIMA (1,0,0).
##################################################################################################################
 
  FNR <- NULL  # false negtive rate
  
  for (sim in 1:5000){    # k is simulation time for IC data
    
    set.seed(1999+sim)
    e <- arima.sim(list(order = c(1,0,0),ar=0.5), n = 365, sd = sqrt(1))
  
    # observed counts
    counts <- ceiling(baseline + e)
    rate <- counts/pop
  
    # fit nonparametric regression for IC data
    span <- loess.wrapper(seq(1:length(rate)),rate, span.vals = seq(0.2, 0.5, by = 0.01), 10)$span
    IC.loess.fit <- loess(rate ~ t,span=span)
    detr.data <- rate -  IC.loess.fit$ fitted
    fit.ts <- auto.arima(detr.data, max.p=3, max.q=3, stationary=T, allowdrift=F, ic=c("aicc"))
    order.ts <- arimaorder(fit.ts)
    coeff <- arima(detr.data,order = order.ts,include.mean=F,method = "CSS")$coef
    IC.residuals <- arima(detr.data,order = order.ts, include.mean=F,method = "CSS")$residuals
    ICmean <- mean(IC.residuals) ;  ICsd <- sd(IC.residuals)
  
  
    # phase II online Monitoring
      delta <- 0.001    # shift size
  
      # simulate out-of-control data
      e <- arima.sim(list(order = c(1,0,0),ar=0.5), n = 365, sd = sqrt(1))
      rate.OC <- (baseline + e)/pop + delta
      detrend.OC.sim.rate <- rate.OC - IC.loess.fit$ fitted
      OC.sim.residuals <- Forecast1(detrend.OC.sim.rate, coeff, order.ts)$residuals
      sdt.OC.sim.residuals <- (OC.sim.residuals-ICmean)/ICsd
    
      # CUSUM control chart monitoring
      k<-0.25
      h0<-xcusum.crit(k,500, mu0 = 0, sided = "one")
      C<-NULL
      C0 <- 0; 
      C[1]<-max(0,C0 + sdt.OC.sim.residuals[1]-k);
      for (i in 2:365){
        C[i]<-max(0,C[i-1]+sdt.OC.sim.residuals[i]-k)
      }  

      FNR[sim] <- length(which(C<h0))/365
} 

# The averaged false negative rate
      mean(FNR)
      sd(FNR)