# This R code is to do one day ahead forecasting for different ARMA time series models

Forecast1 <- function(ts,para,order){
  
  l <- length(ts)
  w <- NULL   #residual
  est <- NULL #one-day ahead prediction
  est[1] <- 0
  w[1] <- ts[1]-est[1]
  
  
  # AR(1) model
  if(all(order == c(1,0,0))){

    for (k in 2:l ){
    est[k] <- ts[k-1]*para[1]
    w[k] <- ts[k] - est[k]}

  }else if (all(order == c(0,0,1))){
    
    for (k in 2:l){
      est[k] <- w[k-1]*para[1]
      w[k] <- ts[k] - est[k]}
  
  }else if(all(order == c(1,0,1))){  # AR(1)MA(1) model
    
    for (k in 2:l){
      est[k] <- ts[k-1]*para[1] + w[k-1]*para[2]
      w[k] <- ts[k] - est[k]}
    
  }else if (all(order == c(1,0,2))){  # AR(1)MA(2) model
    
    est[2] <- ts[1]*para[1] + w[1]*para[2]
    w[2] <- ts[2] - est[2]
    
    for (k in 3:l){
      est[k] <- ts[k-1]*para[1] + w[k-1]*para[2] + w[k-2]*para[3]
      w[k] <- ts[k] - est[k]}
    
  }else if (all(order == c(1,0,3))){  # AR(1)MA(3) model
    
    est[2] <- ts[1]*para[1] + w[1]*para[2]
    w[2] <- ts[2] - est[2]
    est[3] <- ts[1]*para[1] + w[1]*para[2] + w[2]*para[3]
    w[3] <- ts[3] - est[3]
    
    for (k in 4:l){
      est[k] <- ts[k-1]*para[1] + w[k-1]*para[2] + w[k-2]*para[3] + w[k-3]*para[4]
      w[k] <- ts[k] - est[k]}
    
  }else if(all(order == c(2,0,0))){  # AR(2) model
    est[2] <- ts[1]*para[1]
    w[2] <- ts[2] - est[2]
    
    for(k in 3:l){
      est[k] <- ts[k-1]*para[1] + ts[k-2]*para[2]
      w[k] <- ts[k] - est[k]}
  
  }else if(all(order == c(2,0,1))){  # AR(2)MA(1) model
    est[2] <- ts[1]*para[1] + w[1]*para[3]   
    w[2] <- ts[2] - est[2]
    
    for(k in 3:l){
      est[k] <-ts[k-1]*para[1] + ts[k-2]*para[2] + w[k-1]*para[3]
      w[k] <- ts[k] - est[k]}
    
  }else if(all(order == c(2,0,2))){ # AR(2)MA(2) model
    est[2] <- ts[1]*para[1] + w[1]*para[3]   
    w[2] <- ts[2] - est[2]
    
    for(k in 3:l){
      est[k] <-ts[k-1]*para[1] + ts[k-2]*para[2] + w[k-1]*para[3] + w[k-2]*para[4]
      w[k] <- ts[k] - est[k]}
    
  }else if (all(order == c(2,0,3))){ # AR(2)MA(3) model
    est[2] <- ts[1]*para[1] + w[1]*para[3]   
    w[2] <- ts[2] - est[2]
    est[3] <- ts[2]*para[2] + ts[1]*para[2] + w[2]*para[3] + w[1]*para[4]
    w[3] <- ts[3] - est[3]
    
    for(k in 4:l){
      est[k] <-ts[k-1]*para[1] + ts[k-2]*para[2] + w[k-1]*para[3] + w[k-2]*para[4] + w[k-3]*para[5]
      w[k] <- ts[k] - est[k]}
    
  }else if (all(order == c(3,0,0))){ # AR(3) model
    est[2] <- ts[1]*para[1]
    w[2] <- ts[2] - est[2]
    est[3] <- ts[2]*para[1] + ts[1]*para[2]
    w[3] <- ts[3] - est[3]
    
    for(k in 4:l){
      est[k] <- ts[k-1]*para[1] + ts[k-2]*para[2] + ts[k-3]*para[3]
      w[k] <- ts[k] - est[k]}
  
  }else if (all(order == c(3,0,1))){ # AR(3)MA(1) model
    est[2] <- ts[1]*para[1] + w[1]*para[4]
    w[2] <- ts[2] - est[2]
    est[3] <- ts[2]*para[1] + ts[1]*para[2] + w[2]*para[4]
    w[3] <- ts[3] - est[3]
    
    for(k in 4:l){
      est[k] <- ts[k-1]*para[1] + ts[k-2]*para[2] + ts[k-3]*para[3] + w[k-1]*para[4]
      w[k] <- ts[k] - est[k]}
    
  }else if (all(order == c(3,0,2))){  # AR(3)MA(2) model
    est[2] <- ts[1]*para[1] + w[1]*para[4] 
    w[2] <- ts[2] - est[2]
    est[3] <- ts[2]*para[1] + ts[1]*para[2] + w[2]*para[4] + w[1]*para[5]
    w[3] <- ts[3] - est[3]
    
    for(k in 4:l){
      est[k] <- ts[k-1]*para[1] + ts[k-2]*para[2] + ts[k-3]*para[3] + w[k-1]*para[4] + w[k-2]*para[5]
      w[k] <- ts[k] - est[k]}
    
  }else if (all(order == c(3,0,3))){ # AR(3)MA(3) model
    est[2] <- ts[1]*para[1] + w[1]*para[4] 
    w[2] <- ts[2] - est[2]
    est[3] <- ts[2]*para[1] + ts[1]*para[2] + w[2]*para[4] + w[1]*para[5]
    w[3] <- ts[3] - est[3]
    
    for(k in 4:l){
      est[k] <- ts[k-1]*para[1] + ts[k-2]*para[2] + ts[k-3]*para[3] + w[k-1]*para[4] + w[k-2]*para[5] + w[k-3]*para[6]
      w[k] <- ts[k] - est[k]}
  }

  result<-list(est,w)
  names(result)<-c("est","residuals")
  return(result)
}